The following code and explanations are given by raith in [https://stackoverflow.com/a/57128623](https://stackoverflow.com/a/57128623).

    use v6.d;

    constant $ticks = '```';

    my regex Search {
      $ticks '{raku evaluate=' $<evaluate>=(TRUE|FALSE) '}'
      $<code>=[<!before $ticks> .]*
      $ticks
    }

    sub Replace ($/) {
      "Code:\n" ~ $ticks ~ $<code> ~ $ticks ~
        ($<evaluate> eq 'TRUE'
          ?? "\n\n" ~ 'Output:' ~ "\n" ~ $ticks ~ "\n" ~ Evaluate($<code>) ~ $ticks
          !! '');
    }

    sub Evaluate ($code) {
      my $out; my $*OUT = $*OUT but role { method print (*@args) { $out ~= @args } }
      use MONKEY; my $eval-result = EVAL $code;
      $out // $eval-result ~ "\n"
    }

    spurt
      'example_new.md',
      slurp('example.md')
        .subst: &Search, &Replace, :g;

Explanation
-----------

Starting at the bottom and then working upwards:

  * [The `.subst` method](https://docs.raku.org/routine/subst) substitutes parts of its invocant string that need to be replaced and returns the revised string. `.subst`'s first argument is a matcher; it can be a string, or, as here, a regex -- `&Search`'s `.subst`'s second argument is a replacement; this can also be a string, or, as here, a `Callable` -- `&Replace`. If it's a `Callable` then `.subst` passes the match from the matcher as a [match object](https://docs.raku.org/type/Match)<sup>2</sup> as the first argument to the `Callable`. The `:g` adverb directs `.subst` to do the search/replace repeatedly for as many matches as there are in the invocant string.

  * `slurp` generates a string in one go from a file. No need for `open`, using handles, `close`, etc. Its result in this case becomes the invocant of the `.subst` explained above.

  * `spurt` does the opposite, generating a file in one go from a string, in this case the results of the `slurp(...).subst...` operation.

  * The `Evaluate` routine generates a string that's the output from evaluating the string of code passed to it. To capture the result of evaluation it temporarily modifies [Raku's *STDOUT* variable `$*OUT`](https://docs.raku.org/language/variables#index-entry-%24%2AOUT), redirecting `print`s (and thus also `say`s etc.) to the internal variable `$out` before `EVAL`ing the code. If the `EVAL` results in anything being `print`d to `$out` then that is returned; if not, then the result of the `EVAL` is returned (coerced to a string by the `~`). (A newline is appended in this second scenario but not the first because that is what's needed to get the correctly displayed result given how you've "specified" things by your example.)

  * The `Replace` routine is passed a match object from a call of the `Code` regex. It reconstructs the code section (without the evaluate bit) using the `$<code>` capture. If the `$<evaluate>` capture is `'TRUE'` then it also appends a fresh `Output:` section using the `Evaluate` routine explained above to produce the code's output.

  * The `Code` regex *matches* a code section. It *captures* the `TRUE` or `FALSE` setting from the evaluate directive into a capture named `$<evaluate>` and the code into a capture named `$<code>`.

### Footnotes

  * **1** To pass a routine (a regex is a routine) rather than call it, it must be written *with* a sigil (`&foo`), not without (`foo`).

  * **2** It does this even if the matcher was merely a string!

